import json
import os
import pathlib
import sys
import tempfile

from deppth import deppth, sggpio # not sure why the nested path
from PIL import Image, ImageDraw


entries_file = pathlib.Path(__file__).parent / 'entries.json'
with open(entries_file) as f:
    ENTRIES = json.load(f)


def find_subtextures(pkg_reader):
    for entry in pkg_reader:
        if entry.typeName != 'texture':
            continue
        
        for sub in entry.manifest_entry.subAtlases:
            subname = sub['name']
            if 'aphrodite' in subname.lower():
                if subname in ENTRIES:
                    if ENTRIES[subname]:
                        yield entry, sub
                else:
                    print(f'WARNING: Found new subtexture: {fullpath}. This probably means that Supergiant has added new content to the game. This should probably be re-examined.')


def remove_subtexture(path, sub_atlas):
    img = Image.open(path)
    draw = ImageDraw.Draw(img)

    r = sub_atlas['rect']
    top_left = (r['x'], r['y'])
    bottom_right = (r['x'] + r['width'], r['y'] + r['height'])
    draw.rectangle(xy=(top_left, bottom_right), fill=(0, 0, 0, 0))

    img.save(path)


def process_pkg(pkg_path):
    with tempfile.TemporaryDirectory() as tmp:
        tmp = pathlib.Path(tmp)
        extracted = set()
        print('  Extracting textures')
        with sggpio.PackageWithManifestReader(pkg_path) as reader:
            pkg_tmp = tmp / 'package'
            for entry, sub in find_subtextures(reader):
                entry.manifest_entry.extract(pkg_tmp)
                entry.extract(str(pkg_tmp))

                filename = pkg_tmp / f'textures/atlases/{entry.short_name()}.png'
                remove_subtexture(filename, sub)

        print('  Patching package')
        patch_path = tmp / 'patch.pkg'
        deppth.pack(str(pkg_tmp), str(patch_path))
        deppth.patch(pkg_path, patch_path)


def run():
    if len(sys.argv) > 1:
        steam_dir = sys.argv[1]
    else:
        steam_dir = 'C:/Program Files (x86)/Steam'
    base_dir = pathlib.Path(steam_dir) / 'steamapps/common/Hades/Content/Win/Packages'
    if not base_dir.exists():
        print(f'Could not find Hades game files. Is {steam_dir} the correct location for your Steam library?')
        return 1

    for pkg in 'GUI.pkg', '720p/GUI.pkg', 'BC3/GUI.pkg':
        path = str(base_dir / pkg)
        print('Processing package:', pkg)
        process_pkg(path)
        print()

    print('Finished.')


if __name__ == '__main__':
    run()
